@@ -735,122 +735,154 @@ fn parse_expression_node(
     node: &Node<StrDoc<SupportLang>>,
     parser_result: &mut JavaParseResult,
 ) -> Option<JavaExpression> {
-    // Mark as visited but do not early-return; we still need to parse this node
+    enum ExpressionPart {
+        FieldAccess { member: String },
+        MemberMethodCall { member: String },
+        ArrayAccess,
+        MethodReference { member: String },
+    }
 
-    match node.kind().as_ref() {
-        node_types::IDENTIFIER => {
-            parser_result.visited_nodes.insert(node.node_id());
-            Some(JavaExpression::Identifier {
-                name: node.text().into_owned(),
-            })
+    fn build_expression(
+        mut expression: JavaExpression,
+        parts: &mut Vec<ExpressionPart>,
+    ) -> JavaExpression {
+        while let Some(part) = parts.pop() {
+            expression = match part {
+                ExpressionPart::FieldAccess { member } => JavaExpression::FieldAccess {
+                    target: Box::new(expression),
+                    member,
+                },
+                ExpressionPart::MemberMethodCall { member } => JavaExpression::MemberMethodCall {
+                    target: Box::new(expression),
+                    member,
+                },
+                ExpressionPart::ArrayAccess => JavaExpression::ArrayAccess {
+                    target: Box::new(expression),
+                },
+                ExpressionPart::MethodReference { member } => JavaExpression::MethodReference {
+                    target: Box::new(expression),
+                    member,
+                },
+            };
         }
-        node_types::THIS => {
-            parser_result.visited_nodes.insert(node.node_id());
-            Some(JavaExpression::This)
-        }
-        node_types::SUPER => {
-            parser_result.visited_nodes.insert(node.node_id());
-            Some(JavaExpression::Super)
-        }
-        node_types::FIELD_ACCESS => {
-            parser_result.visited_nodes.insert(node.node_id());
-            let target = node
-                .field("object")
-                .map(|n| {
-                    Box::new(
-                        parse_expression_node(&n, parser_result).unwrap_or(JavaExpression::Literal),
-                    )
-                })
-                .unwrap_or_else(|| Box::new(JavaExpression::Literal));
 
-            let member = node
-                .field("field")
-                .map(|n| n.text().into_owned())
-                .unwrap_or_default();
+        expression
+    }
 
-            Some(JavaExpression::FieldAccess { target, member })
-        }
-        node_types::METHOD_INVOCATION => {
-            parser_result.visited_nodes.insert(node.node_id());
-            let target = node.field("object");
+    let mut parts = Vec::new();
+    let mut current = node.clone();
 
-            if let Some(target_node) = target {
-                let target = Box::new(
-                    parse_expression_node(&target_node, parser_result)
-                        .unwrap_or(JavaExpression::Literal),
-                );
+    loop {
+        parser_result.visited_nodes.insert(current.node_id());
 
-                let member = node
-                    .field("name")
+        match current.kind().as_ref() {
+            node_types::IDENTIFIER => {
+                let expression = JavaExpression::Identifier {
+                    name: current.text().into_owned(),
+                };
+
+                return Some(build_expression(expression, &mut parts));
+            }
+            node_types::THIS => {
+                return Some(build_expression(JavaExpression::This, &mut parts));
+            }
+            node_types::SUPER => {
+                return Some(build_expression(JavaExpression::Super, &mut parts));
+            }
+            node_types::FIELD_ACCESS => {
+                let member = current
+                    .field("field")
                     .map(|n| n.text().into_owned())
                     .unwrap_or_default();
 
-                Some(JavaExpression::MemberMethodCall { target, member })
-            } else {
-                let name = node
+                parts.push(ExpressionPart::FieldAccess { member });
+
+                if let Some(object_node) = current.field("object") {
+                    current = object_node;
+                    continue;
+                }
+
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
+            }
+            node_types::METHOD_INVOCATION => {
+                if let Some(target_node) = current.field("object") {
+                    let member = current
+                        .field("name")
+                        .map(|n| n.text().into_owned())
+                        .unwrap_or_default();
+
+                    parts.push(ExpressionPart::MemberMethodCall { member });
+                    current = target_node;
+                    continue;
+                }
+
+                let name = current
                     .field("name")
                     .map(|n| n.text().into_owned())
-                    .unwrap_or_else(|| node.text().into_owned());
+                    .unwrap_or_else(|| current.text().into_owned());
 
-                Some(JavaExpression::MethodCall { name })
+                return Some(build_expression(
+                    JavaExpression::MethodCall { name },
+                    &mut parts,
+                ));
+            }
+            node_types::ARRAY_ACCESS => {
+                parts.push(ExpressionPart::ArrayAccess);
+
+                if let Some(array_node) = current.field("array") {
+                    current = array_node;
+                    continue;
+                }
+
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
+            }
+            node_types::METHOD_REFERENCE => {
+                let member = current
+                    .child(2) // Child 1 is the '::'
+                    .map(|n| n.text().into_owned())
+                    .unwrap_or_default();
+
+                parts.push(ExpressionPart::MethodReference { member });
+
+                if let Some(target_node) = current.child(0) {
+                    current = target_node;
+                    continue;
+                }
+
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
+            }
+            node_types::OBJECT_CREATION_EXPRESSION => {
+                if let Some(type_node) = current.field("type")
+                    && let Some(java_type) = parse_type_node(&type_node)
+                {
+                    return Some(build_expression(
+                        JavaExpression::ObjectCreation {
+                            target: Box::new(java_type),
+                        },
+                        &mut parts,
+                    ));
+                }
+
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
+            }
+            node_types::ARRAY_CREATION_EXPRESSION => {
+                if let Some(type_node) = current.field("type")
+                    && let Some(java_type) = parse_type_node(&type_node)
+                {
+                    return Some(build_expression(
+                        JavaExpression::ArrayCreation {
+                            target: Box::new(java_type),
+                        },
+                        &mut parts,
+                    ));
+                }
+
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
+            }
+            _ => {
+                return Some(build_expression(JavaExpression::Literal, &mut parts));
             }
         }
-        node_types::ARRAY_ACCESS => {
-            parser_result.visited_nodes.insert(node.node_id());
-            let target = node
-                .field("array")
-                .map(|n| {
-                    Box::new(
-                        parse_expression_node(&n, parser_result).unwrap_or(JavaExpression::Literal),
-                    )
-                })
-                .unwrap_or_else(|| Box::new(JavaExpression::Literal));
-
-            Some(JavaExpression::ArrayAccess { target })
-        }
-        node_types::METHOD_REFERENCE => {
-            parser_result.visited_nodes.insert(node.node_id());
-            let target = node
-                .child(0)
-                .map(|n| {
-                    Box::new(
-                        parse_expression_node(&n, parser_result).unwrap_or(JavaExpression::Literal),
-                    )
-                })
-                .unwrap_or_else(|| Box::new(JavaExpression::Literal));
-
-            let member = node
-                .child(2) // Child 1 is the '::'
-                .map(|n| n.text().into_owned())
-                .unwrap_or_default();
-
-            Some(JavaExpression::MethodReference { target, member })
-        }
-        node_types::OBJECT_CREATION_EXPRESSION => {
-            parser_result.visited_nodes.insert(node.node_id());
-            if let Some(type_node) = node.field("type")
-                && let Some(java_type) = parse_type_node(&type_node)
-            {
-                return Some(JavaExpression::ObjectCreation {
-                    target: Box::new(java_type),
-                });
-            }
-
-            Some(JavaExpression::Literal)
-        }
-        node_types::ARRAY_CREATION_EXPRESSION => {
-            parser_result.visited_nodes.insert(node.node_id());
-            if let Some(type_node) = node.field("type")
-                && let Some(java_type) = parse_type_node(&type_node)
-            {
-                return Some(JavaExpression::ArrayCreation {
-                    target: Box::new(java_type),
-                });
-            }
-
-            Some(JavaExpression::Literal)
-        }
-        _ => Some(JavaExpression::Literal),
     }
 }
 
